#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Schaff Trend Cycle
    /// </summary>
    [Description("Schaff Trend Cycle")]
    public class ZiSchaffTrendCycle : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int periodo = 14; // Default setting for Periodo
            private int mACDSlow = 26; // Default setting for MACDLow
            private int mACDFast = 12; // Default setting for MACDFast
			private int mACDSmooth = 9; // Default setting for MACDFast
            private double factor = 0.5; // Default setting for Factor
        // User defined variables (add any user defined variables below)
			private DataSeries sFrac1;
			private DataSeries sFrac2;
			private DataSeries sMACD;
			private DataSeries sPF;
			private DataSeries sPFF;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "Plot0"));
			Add(new Line(Color.LightBlue, 100.00, "banda100"));
			Add(new Line(Color.LightBlue, 70.00, "bandaUpper"));
			Add(new Line(Color.LightBlue, 30.00, "bandaLower"));
			Add(new Line(Color.LightBlue, 0.00, "banda0"));
			
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			
			sFrac1 = new DataSeries(this);
			sFrac2 = new DataSeries(this);
			sMACD = new DataSeries(this);
			sPF = new DataSeries(this);
			sPFF = new DataSeries(this);
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			sMACD.Set( 0, MACD( Close, MACDFast, MACDSlow, MACDSmooth )[0] );
			
			if ( CurrentBar < Periodo )
				return;
			
			double value1 = MIN( sMACD, Periodo )[0];
			double value2 = MAX( sMACD, Periodo )[0] - value1;			
			sFrac1.Set(0, value2 > 0 ? 100 * ( sMACD[0] - value1 ) / value2 : sFrac1[1] );			
			sPF.Set(0, CurrentBar <= 1 ? sFrac1[0] : sPF[1] + ( Factor * ( sFrac1[0] - sPF[1] ) ) );
			
			double value3 = MIN( sPF, Periodo )[0];
			double value4 = MAX( sPF, Periodo )[0] - value3;
			sFrac2.Set(0, value4 > 0 ? 100 * ( sPF[0] - value3 ) / value4 : sFrac2[1]);
			sPFF.Set(0, CurrentBar <= 1 ? sFrac2[0] : sPFF[1] + ( Factor * ( sFrac2[0] - sPFF[1] ) ) );
			
            Plot0.Set(0, sPFF[0]);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

        [Description("Período")]
        [Category("Parameters")]
        public int Periodo
        {
            get { return periodo; }
            set { periodo = Math.Max(1, value); }
        }

        [Description("MACD Low")]
        [Category("Parameters")]
        public int MACDSlow
        {
            get { return mACDSlow; }
            set { mACDSlow = Math.Max(1, value); }
        }

        [Description("MACD Fast")]
        [Category("Parameters")]
        public int MACDFast
        {
            get { return mACDFast; }
            set { mACDFast = Math.Max(1, value); }
        }

		[Description("MACD Smooth")]
        [Category("Parameters")]
        public int MACDSmooth
        {
            get { return mACDSmooth; }
            set { mACDSmooth = Math.Max(1, value); }
        }
		
        [Description("Factor")]
        [Category("Parameters")]
        public double Factor
        {
            get { return factor; }
            set { factor = Math.Max(0, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ZiSchaffTrendCycle[] cacheZiSchaffTrendCycle = null;

        private static ZiSchaffTrendCycle checkZiSchaffTrendCycle = new ZiSchaffTrendCycle();

        /// <summary>
        /// Schaff Trend Cycle
        /// </summary>
        /// <returns></returns>
        public ZiSchaffTrendCycle ZiSchaffTrendCycle(double factor, int mACDFast, int mACDSlow, int mACDSmooth, int periodo)
        {
            return ZiSchaffTrendCycle(Input, factor, mACDFast, mACDSlow, mACDSmooth, periodo);
        }

        /// <summary>
        /// Schaff Trend Cycle
        /// </summary>
        /// <returns></returns>
        public ZiSchaffTrendCycle ZiSchaffTrendCycle(Data.IDataSeries input, double factor, int mACDFast, int mACDSlow, int mACDSmooth, int periodo)
        {
            checkZiSchaffTrendCycle.Factor = factor;
            factor = checkZiSchaffTrendCycle.Factor;
            checkZiSchaffTrendCycle.MACDFast = mACDFast;
            mACDFast = checkZiSchaffTrendCycle.MACDFast;
            checkZiSchaffTrendCycle.MACDSlow = mACDSlow;
            mACDSlow = checkZiSchaffTrendCycle.MACDSlow;
            checkZiSchaffTrendCycle.MACDSmooth = mACDSmooth;
            mACDSmooth = checkZiSchaffTrendCycle.MACDSmooth;
            checkZiSchaffTrendCycle.Periodo = periodo;
            periodo = checkZiSchaffTrendCycle.Periodo;

            if (cacheZiSchaffTrendCycle != null)
                for (int idx = 0; idx < cacheZiSchaffTrendCycle.Length; idx++)
                    if (Math.Abs(cacheZiSchaffTrendCycle[idx].Factor - factor) <= double.Epsilon && cacheZiSchaffTrendCycle[idx].MACDFast == mACDFast && cacheZiSchaffTrendCycle[idx].MACDSlow == mACDSlow && cacheZiSchaffTrendCycle[idx].MACDSmooth == mACDSmooth && cacheZiSchaffTrendCycle[idx].Periodo == periodo && cacheZiSchaffTrendCycle[idx].EqualsInput(input))
                        return cacheZiSchaffTrendCycle[idx];

            ZiSchaffTrendCycle indicator = new ZiSchaffTrendCycle();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Factor = factor;
            indicator.MACDFast = mACDFast;
            indicator.MACDSlow = mACDSlow;
            indicator.MACDSmooth = mACDSmooth;
            indicator.Periodo = periodo;
            indicator.SetUp();

            ZiSchaffTrendCycle[] tmp = new ZiSchaffTrendCycle[cacheZiSchaffTrendCycle == null ? 1 : cacheZiSchaffTrendCycle.Length + 1];
            if (cacheZiSchaffTrendCycle != null)
                cacheZiSchaffTrendCycle.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheZiSchaffTrendCycle = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Schaff Trend Cycle
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ZiSchaffTrendCycle ZiSchaffTrendCycle(double factor, int mACDFast, int mACDSlow, int mACDSmooth, int periodo)
        {
            return _indicator.ZiSchaffTrendCycle(Input, factor, mACDFast, mACDSlow, mACDSmooth, periodo);
        }

        /// <summary>
        /// Schaff Trend Cycle
        /// </summary>
        /// <returns></returns>
        public Indicator.ZiSchaffTrendCycle ZiSchaffTrendCycle(Data.IDataSeries input, double factor, int mACDFast, int mACDSlow, int mACDSmooth, int periodo)
        {
            return _indicator.ZiSchaffTrendCycle(input, factor, mACDFast, mACDSlow, mACDSmooth, periodo);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Schaff Trend Cycle
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ZiSchaffTrendCycle ZiSchaffTrendCycle(double factor, int mACDFast, int mACDSlow, int mACDSmooth, int periodo)
        {
            return _indicator.ZiSchaffTrendCycle(Input, factor, mACDFast, mACDSlow, mACDSmooth, periodo);
        }

        /// <summary>
        /// Schaff Trend Cycle
        /// </summary>
        /// <returns></returns>
        public Indicator.ZiSchaffTrendCycle ZiSchaffTrendCycle(Data.IDataSeries input, double factor, int mACDFast, int mACDSlow, int mACDSmooth, int periodo)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ZiSchaffTrendCycle(input, factor, mACDFast, mACDSlow, mACDSmooth, periodo);
        }

    }
}
#endregion
